/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2 
		(If your project is not GPL, then a fee is due - see license on website)
    Website: http://www.csse.monash.edu.au/~bren/JSX
    Website: http://freshmeat.net/projects/jsx
    List (can read without subscribing, need to join to post):
      http://groups.yahoo.com/group/JSX-ideas/messages
		Commercial licensing enquiries only:
			bren@mail.csse.monash.edu.au */


/** ===========================================================================
	* ObjOut - (could be called XMLObjectOutputStream)
	*	======
	* Output/serialization.
	* There are many possible calls to the Serialization API; but they are
	* all callbacks, from within writeObject(), and so in a sense are invisible.
	**/

package JSX;
//package ObjOutDev;
//import JSX.*;	//use this, not package, if trully a "wrapper"
import java.io.*;	//Stream
import java.util.*;	//Vector eg

public class ObjOut extends ObjectOutputStream {
	final boolean DEBUG = false;
	//final boolean DEBUG = true;

	XMLSerialize s;	//new, for thread-safe version
		//package, so can extract from within PutField...
	public ObjOut(PrintWriter out) throws IOException {
		super();
		s = new XMLSerialize(out);
			//this.out = out; //now maintained within XMLSerialize
		s.setArg(new Object[] {this});	//call-back (best: all in one)
	}
	public ObjOut(PrintWriter out, Config cfg) throws IOException {
		super();
		s = new XMLSerialize(out, cfg);
			//this.out = out; //now maintained within XMLSerialize
		s.setArg(new Object[] {this});	//call-back (best: all in one)
	}

//replaced OutputStreamWriter with Writer (suggested by /lim/)
	public ObjOut(Writer out) throws IOException {
		this(new PrintWriter(out, true));	//so don't need to close it...
	}
	public ObjOut(Writer out, boolean autoFlush) throws IOException {
		this(new PrintWriter(out, autoFlush));
	}
	public ObjOut(Writer out, boolean autoFlush, Config cfg) throws IOException {
		this(new PrintWriter(out, autoFlush), cfg);
	}
	public ObjOut(boolean areAliasesUsed, Writer out) throws IOException {	//links to above
		this(out, true);	//autoFlush by defualt, nice for demo
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut(boolean areAliasesUsed, Writer out, Config cfg) throws IOException {	//links to above
		this(out, true, cfg);	//autoFlush by defualt, nice for demo
		s.areAliasesUsed = areAliasesUsed;
	}

	public ObjOut(OutputStream out, boolean autoFlush) throws IOException {
		this(new OutputStreamWriter(out), autoFlush);	//pass on autoFlush switch
	}
	public ObjOut(OutputStream out, boolean autoFlush, Config cfg) throws IOException {
		this(new OutputStreamWriter(out), autoFlush, cfg);	//pass on autoFlush switch
	}

		//to avoid conflict of sig with above.  Need to rationalize this con API!
	public ObjOut(boolean areAliasesUsed, OutputStream out) throws IOException {	//links to above...
		this(out, true);	//autoFlush by default
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut(OutputStream out) throws IOException {	//links to above...
		this(out, true);	//autoFlush by default
	}
	public ObjOut(OutputStream out, Config cfg) throws IOException {	//links to above...
		this(out, true, cfg);	//autoFlush by default
	}
	//private boolean areAliasesUsed=true; //default "on"; set in constructor


	//protected ObjOut() throws IOException, SecurityException {
	public ObjOut(boolean areAliasesUsed) throws IOException, SecurityException {
		this(System.out);		//NB: of class "OutputStream"
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut(boolean areAliasesUsed, Config cfg) throws IOException, SecurityException {
		this(System.out, cfg);		//NB: of class "OutputStream"
		s.areAliasesUsed = areAliasesUsed;
	}

	public ObjOut() throws IOException, SecurityException {
		this(System.out);		//NB: of class "OutputStream"
	}
	public ObjOut(Config cfg) throws IOException, SecurityException {
		this(System.out, cfg);		//NB: of class "OutputStream"
	}



/* ObjectOutputStream API */
	public void reset() { s.reset(); }
	public void flush() { s.flush(); }
	public void close() { s.close(); }

	protected void writeObjectOverride(Object obj) throws IOException {
		//s.serialize(obj);
		if (DEBUG) System.err.println("writeObject: " +obj);
		s.serialize(obj);	//fine; wrapper APIs must dup a little
		s.flush();
		//we flush automatically for when one whole object graph is
		//written; but it *might* not be what users want for multiple objects...
		//We could just implement it, and add a switch if people complain.
	}
		//NB: *must* override, or OOS's version is called
	public void defaultWriteObject() throws IOException {
		if (DEBUG) System.err.println("defaultWriteObject() - need to pass in");
		s.defaultWriteObject();	//just "return" a flag
	}

/**
	* PutField support (for symbolic evolution of objects)
	* --------
	* Q: why isn't PutField an interface?  It an abstract class because (at one
	* time) it had a public constructor?
	**/
	PutField currentPutField = null;	//NOT ACCOUNT FOR NESTED FIELDS - needed?
		//Imagine two objects in the PutField.  First one has writeObject which
		//is called, which uses PutField, thus replacing currentPutField...
		//does JOS suffer from this problem?
		//It would seem to - it is implemented same as here (but "currentPutFields")

	public PutField putFields() {
		if (currentPutField!=null) {	//a fake check for stack overflow
			if (DEBUG) System.err.println("about to overwrite the currentPutField... need a stack");
			//Because the object is called, the *field* holding it is free; and we
			//don't need to stack. It is because we actually *call* the object - I
			//guess this puts all its data on the stack, too... ;-)");

		}
				// use "new this.s.PutFieldImpl()" so non-static & can access outer
		return currentPutField = this.s.new PutFieldImpl();//a fake "push"
		//will need to stack this - how does JOS do it?
	}
	public void writeFields() throws IOException {	//redundant: also do PutField.write(OOS);
		currentPutField.write(this);	//need a stack (for nested PutFields)
			//in a proper implementation, this would write out the object, 
			//and invoke its readObject(), which could create a new a OutField
			//object, displacing the "currentPutFields" field... need to use a stack

		currentPutField = null;	//a fake "pop"
		if (DEBUG) System.err.println("writeFields() fields and values buffered.");
	}


	public void write(byte[] b) throws IOException {
		writeObject(b);	//handle it as an object (why not?)
	}
/** don't use this! Meant to be efficient, but horrifically ineff! ;-) **/
	public void write(byte[] b, int off, int len) throws IOException {
		byte[] b2 = new byte[len];
		System.arraycopy(b, off, b2, 0, len);
		writeObject(b2);	//handle it as an object (why not?)
	}
	public void write(int b) throws IOException {	// was missed, until 8Oct2002
		s.writePrim(b+"");
	}
	public void writeInt(int data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeFloat(float data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeBoolean(boolean data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeByte(int data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeBytes(String data) throws IOException {
		//System.err.println("Should this be written as an object (so alias is used?)");
			//The above comment has never been reported
		writeObject(data);	//handle it as an object (why not?)
	}
	public void writeChar(int data) throws IOException {	//"int" matches OOS sig
		s.writePrim(data+"");		//Writes out as a decimal number
		//s.writePrim(ParseUtilities.encodeXML( ((char)data)+"");
		//s.writePrim(((char)data)+"");		//Excellent bug diagnosis by Teck Hooi!
		//easier for back-compatibility, to simply write a char as a number,
		//instead of the complex escaping.
		//The problem is the XML issue of representing the null (char) 0 and
		//control characters.
	}
	public void writeChars(String data) throws IOException {
		System.err.println("writeChars() is not properly implemented");
		//System.err.println("Should this be written as an object (so alias is used?)");
		writeObject(data);	//handle it as an object (why not?)
			// write with or without aliases?  How to tell?
	}
	public void writeDouble(double data) throws IOException {
		s.writePrim(data+"");
	}
	//Float - see above
	//Int - see above
		//writeLong() is thanks Andy Thomas (who found Date didn't work).
		//He's the first successful JSX Challenger. Wed, 10 Jan 2001 22:33:57 +0000
		//James Skinnell found it, too (through java.sql.Timestamp not working).
		//He's the second successful JSX Challenger. Thu Jan 11 08:06:28 2001
	public void writeLong(long data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeShort(int data) throws IOException {	//"int" matches OOS
		s.writePrim(data+"");
	}
		/**
		 	* writeUTF(str)
			* ------------
			* writeUTF specifies the data format, which is not relevant to us, as
			* we use an XML format.  However, it does have the side-effect of
			* treating a String 100% as a primitive, in the sense that there is
			* no aliasing (confirmed with Java's own Serialization: "java TestSuite e"
			**/
	public void writeUTF(String data) throws IOException {
  	//System.err.println("writePrim() is correct, not writeObject()");
		s.writePrim(data+"");
/*
		writeObject(data);	//handle it as an object (why not?)
*/
	}
	static class Non_method_innerclassTest {}
	
	
	/**
		* setFormatted(boolean)
		* -----------------
		* Turns XML Output formatting on or off..
		* Formatted XML is spaced and indented
		* Non-formatted XML is contained without indentation or whitespaces between
		* tags, or any CR/LF
		* Added by Gil 
		**/
	public void setFormatted(boolean formatted) {
    s.setFormatted(formatted);
	}
	

	public static void main(String[] args) throws Exception {
		Config c = new Config().aliasID(false);	// turn aliases off for eg
		ObjOut out = new ObjOut(c);
		Object obj;
		if (args.length>0) {
			Class clazz = Class.forName(args[0]);
			// invoke no-arg constructor
					// was: obj = clazz.newInstance();
			java.lang.reflect.Constructor cons = clazz.getDeclaredConstructor(new Class[] {});
			cons.setAccessible(true);	// necessary
			obj = cons.newInstance(new Object[] {});	// exceptions at main()
		} else {
			
			obj = new Person();
/*
			obj = new MyTyreClass("wheels on my car",
				new Wheel("Michelin",  new MyDate(31,10,1997)),
				new Wheel("Michelin",  new MyDate(31,10,1997)),
				new Wheel("Michelin",  new MyDate(2,3,1999)),
				new Wheel("Michelin",  new MyDate(14,9,2000))
			);
*/
		}
/*
			obj = new java.util.Hashtable();
			((Hashtable)obj).put("a", "alpha");
			((Hashtable)obj).put("b", "beta");
			((Hashtable)obj).put("g", "gamma");
*/
		out.writeObject(obj);		//so simple to use!
	}
}
