/** (c) Brendan Macmillan 2002, licensed under GNU's GPL 2 
		(If your project is not GPL, then a fee is due - see license on website)
    Website: http://www.csse.monash.edu.au/~bren/JSX
    Website: http://freshmeat.net/projects/jsx
    List (can read without subscribing, need to join to post):
      http://groups.yahoo.com/group/JSX-ideas/messages
		Commercial licensing enquiries only:
			bren@objectlight.com */


package JSX.magic;
import java.util.*;

/** Select "under the hood" connections to private implementation of Java,
	* based on spec version and impl vendor properties, and without compile-time
	* dependancies
	* Note: other's vendors implementations may require different connections
	*  - see list at end.  This approach can accommodate them.
	*
	* Issue: how to arrange the detection and selection...
	*  - If statements?
	*  - A hash?
	*  - subclassing? (ie: first: 1.2, 1.3 or 1.4; then specific vendors; then...)
	*  - a matrix?
	*  - decision graph?
	* NB: Efficiency is irrelevant (not inner loop); clarity is important.
	**/
public class MagicClassFactory {
	public static MagicClassI newInstance() {
		Properties p = System.getProperties();

	// Idea: make up an inner class, with properties to match; and a result to
	// give. Run through an array of these.  By factoring out the superstructure,
	// it should be easier to see that the correct things are linked up.
	// In fact, it might be better to do it by order, because it is easy to
	// pick out mistakes when arranged in columns...
	// Comments are most welcome!!

		String specVersion = p.getProperty("java.specification.version");
    String implVersion = p.getProperty("java.version");
    String implVendor = p.getProperty("java.vendor");

		String magicName = null; // flag error if not set! "JSX.magic.MagicClass13";
		if (implVendor.equals("Sun Microsystems Inc.") && ( specVersion.equals("1.2") || specVersion.equals("1.3") ) )
			magicName = "JSX.magic.MagicClass13";	// explicit (even tho == default)

		if (implVendor.equals("Sun Microsystems Inc.") && specVersion.equals("1.4"))
			magicName = "JSX.magic.MagicClass14";	// specific

		if (implVendor.equals("IBM Corporation") && specVersion.equals("1.3") && implVersion.equals("1.3.0 subset"))
			magicName = "JSX.magic.MagicClass13_j9";	// "j9" or "J9"?

		if (implVendor.equals("Blackdown Java-Linux Team") && specVersion.equals("1.3") && implVersion.equals("1.3.1"))
			magicName = "JSX.magic.MagicClass13";

		if (implVendor.equals("Blackdown Java-Linux Team") && specVersion.equals("1.4") && implVersion.equals("1.4.1"))
			magicName = "JSX.magic.MagicClass14";	// thanks to Egon

		if (implVendor.equals("Apple Computer, Inc.") && specVersion.equals("1.3") && implVersion.equals("1.3.1"))
			magicName = "JSX.magic.MagicClass13";

		if (magicName==null) {
			System.err.println("---ATTENTION!---  JSX could not recognize your implementation of java, which is:");
			System.err.println("\timplVendor='"+implVendor+"'");
			System.err.println("\tspecVersion='"+specVersion+"'");
			System.err.println("\timplVersion='"+implVersion+"'");

			String tryingVersion;	// to provoke compiler error if not set
			if (specVersion.equals("1.2") || specVersion.equals("1.3")) {
				tryingVersion = "1.3";
				magicName = "JSX.magic.MagicClass13";	// for both 1.2 and 1.3
			} else if (specVersion.equals("1.4")) {
				tryingVersion = "1.4";
				magicName = "JSX.magic.MagicClass14";
			} else {
				tryingVersion = "1.4"; // default to 1.4
				magicName = "JSX.magic.MagicClass14";
			}
			System.err.println("In the meantime, JSX will try the standard implementation for Java "+tryingVersion+" - which will probably work");
			System.err.println("Please post the above information to: jsx-ideas@yahoogroups.com");
			System.err.println(" - in particular, please state whether the standard implementation worked or not, for both writing and reading.");
		}
/*
		if (specVersion.equals("1.2") || specVersion.equals("1.3"))
			magicName = "JSX.magic.MagicClass13";
		else if (specVersion.equals("1.4"))
			magicName = "JSX.magic.MagicClass14";
		else	//default
			magicName = "JSX.magic.MagicClass13";
*/

/*
OTIs j9 jvm (David Tottey)
	java.specification.version=1.3
	java.specification.vendor=Sun Microsystems Inc.
	java.version=1.3.0 subset
	java.vendor=IBM Corporation
*/
// Nice if this message came up, when a failure encountered:
		// If this fails, please report to report to bren@mail.csse.monash.edu.au,
		// with specVersion and implVendor.
		try {
			return (MagicClassI) Class.forName(magicName).newInstance();
		} catch (InstantiationException cantMake) {
			throw new InternalError("Can't instantiate "+magicName);
		} catch (ClassNotFoundException noClass) {
			throw new InternalError("Can't find "+magicName);
		} catch (IllegalAccessException noAccess) {
			throw new InternalError("Not permitted to access "+magicName);
		}
	}
}




/*
With gcj 3.2 and JSX 1.0.3.1 (*)

8<---------------------------------------------------
$ ./a.out 
java.specification.version=1.3
java.specification.vendor=Sun Microsystems Inc.

java.version=3.2
java.vendor=Free Software Foundation, Inc.
8<---------------------------------------------------

(*) I used a small patch to compile JSX/ObjIn.java

(And as I said previously for gcj 3.1.1,
> /home/DEVENV/JSX1.0.3.0/XSLTimport.java
> [SNIP] javax.xml from JDK1.4 not available in gcj 3.1.1
> 6 errors
)

With orp 1.0.9,
8<---------------------------------------------------
$ orp TestProperties
java.specification.version=null
java.specification.vendor=null

java.version=GNU Classpath 0.02
java.vendor=Intel MRL
8<---------------------------------------------------

With kissme 0.21,
8<---------------------------------------------------
$ kissme TestProperties      
java.specification.version=null
java.specification.vendor=null

java.version=1.2
java.vendor=John Leuner
8<---------------------------------------------------

With kaffe 1.0.5,
8<---------------------------------------------------
$ /usr/bin/java TestProperties 
java.specification.version=1.2
java.specification.vendor=Sun Microsystems Inc.

java.version=1.0.5
java.vendor=Transvirtual Technologies, Inc.
8<---------------------------------------------------

Not yet tested with SableVM.

-- 
Benot Sibaud
http://oumph.free.fr
*/
