_Y.use("gallery-accordion", "console", "node", function(Y) {
	
	var TOPIC_BODY_TEMPLATE =
		"<div id=\'topic-{{ID}}\' class=\'topic-entry\'>\n" + 
		"	<div class=\'content-legend\'>{{PROMPT}}</div>\n" + 
		"	<div class=\'content-body\' id=\'prompts-{{ID}}\'></div>\n" + 
		"	\n" + 
		"	<div class=\'content-legend\'>{{CREATE_CARD}}</div>\n" + 
		"	<div class=\'content-body topic-editor\' id=\'editor-{{ID}}\'>\n" + 
		"		<div style=\'width: 70%; float: left;\'>\n" + 
		"			<div style=\'padding-bottom: 5px;\'>\n" + 
		"				<label style=\'float: left; width: 15%;\' for=\'text-{{ID}}\'>{{TEXT}}:</label>\n" + 
		"				<input type=\'text\' id=\'text-{{ID}}\'/>\n" + 
		"			</div>\n" + 
		"			<div style=\'clear: both;\'>\n" + 
		"				<label style=\'float: left; width: 15%;\' for=\'type-{{ID}}\'>{{CARD_TYPE}}:</label>\n" + 
		"				<select id=\'type-{{ID}}\' style=\'\'>\n" + 
		"					<option value=\'metabasicelement\' checked=\'checked\'>{{META_ELEMENT}}</option>\n" + 
		"					<option value=\'activity\'>{{ACTIVITY}}</option>\n" + 
		"					<option value=\'entity\'>{{ENTITY}}</option>\n" + 
		"					<option value=\'role\'>{{ROLE}}</option>\n" + 
		"					<option value=\'basicmodifier\'>{{BASICMODIFIER}}</option>\n" + 
		"				</select>\n" + 
		"			</div>\n" + 
		"		</div>\n" + 
		"		<div style=\'width: 30%; float: right;\' >\n" + 
		"			<input type=\'button\' style=\'width: 100%; height: 100%; font-size: 40px;\' value=\'{{SEND}}\' id=\'send-{{ID}}\'/>\n" + 
		"		</div>\n" + 
		"		<div style=\'clear: both;\'></div>\n" + 
		"	</div>\n" + 
		"	\n" + 
		"	<div id=\'history-{{ID}}\' class=\'history-wrapper history-closed\'>\n" + 
		"		<div class=\'content-legend history-head\' id=\'history-head-{{ID}}\'>\n" + 
		"			{{HISTORY}}\n" + 
		"			<a href=\'#\' class=\'history-icon\'></a>\n" + 
		"			<div style=\'clear: both;\'></div>\n" + 
		"		</div>\n" + 
		"		<div class=\'content-body history-body\' id=\'history-body-{{ID}}\'></div>\n" + 
		"	</div>\n" + 
		"</div>";

	
	/**
	 * Topic Klasse
	 */
	Topic = function(data) {
		var item = new Y.AccordionItem({
			label: data.topic,
			iconAlwaysVisible: Y.Node.create("<span></span>"),
			expanded: false,
			id: "topic-item-" + data.id,
			contentHeight: {
				method: "auto"
			}
		});
	
		var markup = TOPIC_BODY_TEMPLATE.replace(/{{ID}}/g, data.id);
		markup = localizeTemplate(markup);
		var body = Y.Node.create(markup);
		item.set("bodyContent", body);

		this._data = data;
		this._item = item;
	};
	
	Topic.prototype = {
		_data: null,
		_item: null,
		_cards: [],
		
		getData: function() { return this._data; },
		
		getItem: function() { return this._item; },
		
		getId: function() { return this.getData().id; } ,
		
		update: function(data) {
			this._data = data;
			this._item.set("label", data.topic);
		},
		
		addCard: function(card) {
			this._cards[this._cards.length] = card;
			
			Y.one("#history-body-" + this._data.id).insert(card.getHistoryHtmlElement(), 0);
			this._item.resize();
		},
	
		sendAction: function(e) {
			var id = this._data.id;
			var sendBtn = Y.one("#send-" + id).set("disabled", true);
			var textInput = Y.one("#text-" + id).set("disabled", true);
		
			var text = textInput.get("value");
			var type = Y.one("#type-" + id).get("value");
			
			if (text.replace(/\s/g, "") == "") {
				alert(getLocalizedString("EMPTY_TEXT_ERROR"));
			} else {
				var card = new Card(type, text, id);
				var cfg = {
					method: "POST",
					data: "text=" + text + "&seeme_object=" + type + "&seeme-topic-id=" + id + "&name=nobody",
					on: {
						success: this.handleSuccess,
						failure: this.handleFailure
					},
					arguments: {
						card: card
					},
					context: this
				};
				
				this.addCard(card);
				var req = Y.io("JSONServlet?action=send-message&seeme-message-id=" + card.getId(), cfg);
			}
		
			textInput.set("value", "");
			sendBtn.set("disabled", false);
			textInput.set("disabled", false);
			textInput.focus();
		},
		
		handleFailure: function(ioId, o, ctx) {
			ctx.card.setState('failure');
		},
		
		handleSuccess: function(ioId, o, ctx) {
			ctx.card.setState('ok');
		},
		
		toggleHistoryForTopic: function(e) {
			var h = Y.one("#history-" + this._data.id);
			h.toggleClass("history-closed");
			h.toggleClass("history-opened");
			this.getItem().resize();
			this.focus();
		},
		
		focus: function() {
			Y.one("#text-" + this._data.id).focus();
		},

		onAccordionItemKeyPress: function(e) {
			if (e.keyCode == 13) {
				this.sendAction(e);
			}
		},
		
		initListeners: function() {
			Y.one("#send-" + this._data.id).on('click', this.sendAction, this);
			Y.one("#text-" + this._data.id).on('keypress', this.onAccordionItemKeyPress, this);
			Y.one("#history-head-" + this._data.id).on('click', this.toggleHistoryForTopic, this);
		}
	};
});
